@@ -330,19 +330,6 @@ class ArgparseBuilder:
             advanced=True,
             argtype="filename",
         )
-        self.add_argument(
-            parser,
-            "--numbers_handling",
-            default="default",
-            choices=["default", "all", "none"],
-            help="question numbers handling strategy. "
-            "Default preserves zero questions and numbering "
-            "if the first question has number > 1, omits number otherwise. "
-            "All preserves all numbers, none omits all numbers "
-            "(was default behaviour pre-0.8.0.)",
-            advanced=True,
-            caption="Стратегия обработки номеров вопросов",
-        )
         self.add_argument(
             parser,
             "-v",
@@ -416,16 +403,37 @@ class ArgparseBuilder:
             advanced=True,
             caption="Fix <span> tags",
         )
+        self.add_argument(
+            cmdparse,
+            "--image_prefix",
+            action="store_true",
+            help="make image prefix from filename",
+            advanced=True,
+            caption="Make image prefix from filename",
+        )
         self.add_argument(
             cmdparse,
             "--parsing_engine",
-            choices=["pypandoc", "mammoth_bs_prettify", "mammoth_bs_hard_unwrap", "mammoth"],
+            choices=["pypandoc", "pypandoc_html", "mammoth_bs_prettify", "mammoth_bs_hard_unwrap", "mammoth"],
             default="mammoth",
             help="old html processing behaviour (before v0.5.5). "
             "Sometimes it will yield better results than the new default.",
             advanced=True,
             caption="BeautifulSoup prettify",
         )
+        self.add_argument(
+            cmdparse,
+            "--numbers_handling",
+            default="default",
+            choices=["default", "all", "none"],
+            help="question numbers handling strategy. "
+            "Default preserves zero questions and numbering "
+            "if the first question has number > 1, omits number otherwise. "
+            "All preserves all numbers, none omits all numbers "
+            "(was default behaviour pre-0.8.0.)",
+            advanced=True,
+            caption="Стратегия обработки номеров вопросов",
+        )
 
         cmdcompose = subparsers.add_parser("compose")
         self.add_argument(
