@@ -12,9 +12,12 @@ import shlex
 import logging
 import base64
 import itertools
+import tempfile
+import shutil
 import chardet
 import mammoth
 import pypandoc
+import dashtable
 import bs4
 from bs4 import BeautifulSoup
 from parse import parse
@@ -220,6 +223,18 @@ def chgk_parse(text, defaultauthor=None, regexes=None):
         with codecs.open("debug_1.json", "w", "utf8") as f:
             f.write(json.dumps(structure, ensure_ascii=False, indent=4))
 
+    # hack for https://gitlab.com/peczony/chgksuite/-/issues/23; TODO: make less hacky
+    for i, element in enumerate(structure):
+        if (
+            "дуплет." in element[1].lower().split() or "блиц." in element[1].lower().split()
+            and element[0] != "question" and (i == 0 or structure[i - 1][0] != "question")
+        ):
+            element[0] = "question"
+
+    if debug:
+        with codecs.open("debug_1a.json", "w", "utf8") as f:
+            f.write(json.dumps(structure, ensure_ascii=False, indent=4))
+
     # 2.
 
     merge_y_to_x("question", "answer")
@@ -502,22 +517,41 @@ def chgk_parse_txt(txtfile, encoding=None, defaultauthor="", regexes=None):
     return chgk_parse(text, defaultauthor=defaultauthor, regexes=regexes)
 
 
-def generate_imgname(target_dir, ext):
+def generate_imgname(target_dir, ext, prefix=""):
     imgcounter = 1
     while os.path.isfile(
-        os.path.join(target_dir, "{:03}.{}".format(imgcounter, ext))
+        os.path.join(target_dir, "{}{:03}.{}".format(prefix, imgcounter, ext))
     ):
         imgcounter += 1
-    return "{:03}.{}".format(imgcounter, ext)
+    return "{}{:03}.{}".format(prefix, imgcounter, ext)
+
+
+def ensure_line_breaks(tag):
+    if tag.text:
+        str_ = tag.string or "".join(list(tag.strings))
+        if not str_.startswith("\n"):
+                str_ = "\n" + str_
+        if not str_.endswith("\n"):
+            str_ = str_ + "\n"
+        tag.insert_before(str_)
+    tag.extract()
 
 
 def chgk_parse_docx(docxfile, defaultauthor="", regexes=None, args=None):
     target_dir = os.path.dirname(os.path.abspath(docxfile))
+    if args.image_prefix:
+        bn_for_img = os.path.splitext(os.path.basename(docxfile))[0].replace(" ", "_") + "_"
+    else:
+        bn_for_img = ""
     if args.parsing_engine == "pypandoc":
         txt = pypandoc.convert_file(docxfile, "plain", extra_args=["--wrap=none"])
     else:
-        with open(docxfile, "rb") as docx_file:
-            html = mammoth.convert_to_html(docx_file).value
+        if args.parsing_engine == "pypandoc_html":
+            temp_dir = tempfile.mkdtemp()
+            html = pypandoc.convert_file(docxfile, "html", extra_args=[f"--extract-media={temp_dir}"])
+        else:
+            with open(docxfile, "rb") as docx_file:
+                html = mammoth.convert_to_html(docx_file).value
         input_docx = (
             html
             .replace("</strong><strong>", "")
@@ -534,9 +568,23 @@ def chgk_parse_docx(docxfile, defaultauthor="", regexes=None, args=None):
 
         for tag in bsoup.find_all("style"):
             tag.extract()
+        for tag in bsoup.find_all("img"):
+            if args.parsing_engine == "pypandoc_html":
+                src = tag["src"].replace("$$$UNDERSCORE$$$", "_")
+                _, ext = os.path.splitext(src)
+                imgname = generate_imgname(target_dir, ext[1:], prefix=bn_for_img)
+                shutil.copy(src, os.path.join(target_dir, imgname))
+                imgpath = os.path.basename(imgname)
+            else:
+                imgparse = parse("data:image/{ext};base64,{b64}", tag["src"])
+                imgname = generate_imgname(target_dir, imgparse["ext"], prefix=bn_for_img)
+                with open(os.path.join(target_dir, imgname), "wb") as f:
+                    f.write(base64.b64decode(imgparse["b64"]))
+                imgpath = os.path.basename(imgname)
+            tag.insert_before("(img {})".format(imgpath))
+            tag.extract()
         for tag in bsoup.find_all("p"):
-            if tag.string:
-                tag.string = tag.string + SEP
+            ensure_line_breaks(tag)
         for tag in bsoup.find_all("b"):
             tag.unwrap()
         for tag in bsoup.find_all("strong"):
@@ -552,17 +600,13 @@ def chgk_parse_docx(docxfile, defaultauthor="", regexes=None, args=None):
                 tag.unwrap()
         for h in ["h1", "h2", "h3", "h4"]:
             for tag in bsoup.find_all(h):
-                tag.unwrap()
+                ensure_line_breaks(tag)
         for tag in bsoup.find_all("li"):
             if tag.string:
                 tag.string = "- " + tag.string
-        for tag in bsoup.find_all("img"):
-            imgparse = parse("data:image/{ext};base64,{b64}", tag["src"])
-            imgname = generate_imgname(target_dir, imgparse["ext"])
-            with open(os.path.join(target_dir, imgname), "wb") as f:
-                f.write(base64.b64decode(imgparse["b64"]))
-            imgpath = os.path.basename(imgname)
-            tag.insert_before("(img {})".format(imgpath))
+        for tag in bsoup.find_all("table"):
+            table = dashtable.html2md(str(tag))
+            tag.insert_before(table)
             tag.extract()
         for tag in bsoup.find_all("hr"):
             tag.extract()
@@ -598,9 +642,16 @@ def chgk_parse_docx(docxfile, defaultauthor="", regexes=None, args=None):
                 if isinstance(tag, bs4.element.Tag):
                     tag.unwrap()
             txt = bsoup.prettify()
-        elif args.parsing_engine == "mammoth":
-            html2text_input = str(bsoup)
-            txt = h.handle(html2text_input)
+        # elif args.parsing_engine == "mammoth":
+        #     html2text_input = str(bsoup)
+        #     txt = h.handle(html2text_input)
+        elif args.parsing_engine in ("pypandoc_html", "mammoth"):
+            for tag in bsoup:
+                if isinstance(tag, bs4.element.Tag):
+                    tag.unwrap()
+            txt = str(bsoup)
+    if args.parsing_engine == "pypandoc_html":
+        shutil.rmtree(temp_dir)
 
     txt = (
         txt.replace("\\-", "")
